module Kalendar exposing (getWeekday, printDate, viewCalendar, weekDates)

import Date exposing (Date, Interval(..), weekday)
import Html exposing (Html, div)
import Html.Attributes exposing (style)
import Html.Events exposing (onClick)
import List.Extra exposing (groupsOf)
import String exposing (concat)
import Time exposing (Month(..), Weekday(..))
import Types exposing (..)


weekdayHeader : Html a
weekdayHeader =
    Html.thead
        []
        [ Html.tr
            []
            ([ "Mo", "Di", "Mi", "Do", "Fr", "Sa", "So" ]
                |> List.map
                    (\str ->
                        Html.th
                            [ style "padding" "0.5em"
                            , style "font-weight" "normal"
                            , style "font-style" "italic"
                            , style "color" "gray"
                            ]
                            [ Html.text str
                            ]
                    )
            )
        ]


weekDates : Date -> List Date
weekDates date =
    let
        start =
            Date.fromCalendarDate (Date.year date) (Date.month date) (Date.day date)
                |> Date.floor Date.Monday

        until =
            start |> Date.add Date.Days 7
    in
    Date.range Date.Day 1 start until


monthDates : Int -> Time.Month -> List Date
monthDates year month =
    let
        start =
            Date.fromCalendarDate year month 1
                |> Date.floor Date.Monday

        until =
            start |> Date.add Date.Days 42
    in
    Date.range Date.Day 1 start until


viewMonthTable : Date -> LoadingState -> Html Msg
viewMonthTable target loadingRes =
    let
        weeks =
            monthDates (Date.year target) (Date.month target)
                |> groupsOf 7
    in
    Html.table
        [ style "border-collapse" "collapse"
        , style "text-align" "right"
        ]
        [ weekdayHeader
        , Html.tbody
            []
            (weeks
                |> List.map
                    (\weekdates ->
                        Html.tr
                            []
                            (weekdates
                                |> List.map
                                    (\date ->
                                        let
                                            color =
                                                if Date.month date == Date.month target then
                                                    if getStatus date loadingRes == Free then
                                                        "black"

                                                    else
                                                        "red"

                                                else
                                                    "lightgray"

                                            background =
                                                if date == target then
                                                    "lightskyblue"

                                                else
                                                    "transparent"
                                        in
                                        Html.td
                                            [ style "padding" "0.5em"
                                            , style "background" background
                                            , style "color" color
                                            , onClick (SetDate date)
                                            ]
                                            [ Html.text (Date.day date |> String.fromInt)
                                            ]
                                    )
                            )
                    )
            )
        ]


viewCalendar : Date -> LoadingState -> Html Msg
viewCalendar date loadingRes =
    div
        [ style "padding" "2em"
        , style "font-family" "Helvetica, Arial, san-serif"
        , style "font-size" "16px"
        ]
        [ Html.h2
            [ style "font-size" "16px"
            , style "margin" "0"
            , style "padding" "0 0.5em 2em"
            ]
            [ Html.text (toGermanMonth (Date.month date) ++ (date |> Date.format " yyyy"))
            ]
        , viewMonthTable date loadingRes
        ]


toGermanMonth : Date.Month -> String
toGermanMonth month =
    case month of
        Jan ->
            "Januar"

        Feb ->
            "Februar"

        Mar ->
            "März"

        Apr ->
            "April"

        May ->
            "Mai"

        Jun ->
            "Juni"

        Jul ->
            "Juli"

        Aug ->
            "August"

        Sep ->
            "September"

        Oct ->
            "Oktober"

        Nov ->
            "November"

        Dec ->
            "Dezember"


getWeekday : Date -> String
getWeekday date =
    case weekday date of
        Mon ->
            "Montag"

        Tue ->
            "Dienstag"

        Wed ->
            "Mittwoch"

        Thu ->
            "Donnerstag"

        Fri ->
            "Freitag"

        Sat ->
            "Samstag"

        Sun ->
            "Sonntag"


printDate : Date -> String
printDate date =
    concat
        [ String.fromInt (Date.day date)
        , ". "
        , toGermanMonth (Date.month date)
        , " "
        , String.fromInt (Date.year date)
        ]
