module Main exposing (main)

--import Html.Attributes.Aria exposing (role)

import Browser
import Browser.Dom exposing (Error(..))
import Browser.Navigation as Nav
import Date exposing (Date, Unit(..), add)
import Html exposing (Html, a, button, div, fieldset, figure, footer, h1, h2, h3, input, option, section, select, span, text)
import Html.Attributes exposing (class, href, placeholder, style, type_, value)
import Html.Events exposing (onClick, onDoubleClick, onInput)
import Http
import Json.Decode exposing (Decoder, field, int, list, map, map2, map4, string)
import Kalendar exposing (getWeekday, printDate, viewCalendar, weekDates)
import List.Extra exposing (groupsOf)
import LogoLibary exposing (getLogo)
import Svg
import Svg.Attributes exposing (x, y)
import SystemLibary exposing (..)
import Task
import Time exposing (Month(..))
import Types exposing (..)
import Url


main : Program () Model Msg
main =
    Browser.application
        { init = init
        , view = view
        , update = update
        , subscriptions = subscriptions
        , onUrlRequest = ClickedLink
        , onUrlChange = UrlChanged
        }



-- INIT


init : () -> Url.Url -> Nav.Key -> ( Model, Cmd Msg )
init _ url key =
    ( { modal = Nothing
      , saved =
            { date = "2019-01-01"
            , hour = 17
            , dm = "Josch"
            , system = "Hexxen1733"
            }
      , date = Date.fromCalendarDate 2019 Jan 1
      , status = Unknown
      , loadingRes = None
      , loadingSystem = NoneS
      , key = key
      , url = url
      }
    , Cmd.batch [ Date.today |> Task.perform SetDate, getReservations, getSystems ]
    )


type alias Document msg =
    { title : String
    , body : List (Html msg)
    }



-- UPDATE


update : Msg -> Model -> ( Model, Cmd Msg )
update msg model =
    case msg of
        SetDate date ->
            let
                newModel =
                    updateStatus date model
            in
            ( { newModel | date = date }, Cmd.none )

        OpenModal modalMsg ->
            updateModal modalMsg model

        CloseModal ->
            ( { model | modal = Nothing }, Cmd.none )

        LoadReservations ->
            ( model, getReservations )

        GotReservations result ->
            case result of
                Ok reservation ->
                    ( { model | loadingRes = Success reservation }, Cmd.none )

                Err _ ->
                    ( { model | loadingRes = Failure }, Cmd.none )

        LoadSystems ->
            ( model, getSystems )

        GotSystems result ->
            case result of
                Ok syst ->
                    ( { model | loadingSystem = SuccessS syst }, Cmd.none )

                Err _ ->
                    ( { model | loadingSystem = FailureS }, Cmd.none )

        UrlChanged url ->
            ( { model | url = url }, Cmd.none )

        ClickedLink urlRequest ->
            case urlRequest of
                Browser.Internal url ->
                    case url.fragment of
                        Nothing ->
                            ( model, Nav.pushUrl model.key (Url.toString url) )

                        Just fragment ->
                            ( model, Nav.pushUrl model.key (Url.toString url) )

                Browser.External href ->
                    ( model, Nav.load href )


updateModal : ModalMsg -> Model -> ( Model, Cmd Msg )
updateModal modalMsg model =
    case modalMsg of
        OpenShowCalendar ->
            ( { model | modal = Just ShowCalendar }, Cmd.none )

        OpenShowReservation ->
            ( { model | modal = Just ShowReservation }, Cmd.none )

        OpenInputReservation date ->
            ( { model
                | modal =
                    Just
                        (InputReservation
                            { date = Date.toIsoString date
                            , hour = 17
                            , dm = ""
                            , system = ""
                            }
                            model.loadingSystem
                        )
              }
            , Cmd.none
            )

        Change reservation ->
            --{ model | modal = Just (InputReservation reservation) }
            ( { model | saved = reservation }, Cmd.none )

        -- ToDo save reservation to db
        Add reservation ->
            ( { model | modal = Nothing }, Cmd.none )

        OpenInputSystem ->
            ( { model
                | modal =
                    Just
                        (InputSystem
                            { name = ""
                            , url = ""
                            }
                        )
              }
            , Cmd.none
            )

        OpenShowSystem system ->
            ( { model | modal = Just (ShowSystem system) }, Cmd.none )

        ChangeS system ->
            --{ model | modal = Just (InputReservation reservation) }
            ( { model | modal = Nothing }, Cmd.none )

        -- ToDo save system to db
        AddS system ->
            ( { model | modal = Nothing }, Cmd.none )


updateStatus : Date -> Model -> Model
updateStatus date model =
    case model.loadingRes of
        Loading ->
            { model | status = Free }

        Failure ->
            { model | status = Unknown }

        None ->
            { model | status = Free }

        Success reservations ->
            let
                result =
                    List.Extra.find (\r -> r.date == Date.toIsoString date) reservations
            in
            case result of
                Nothing ->
                    { model | status = Free }

                Just res ->
                    { model | status = Taken, saved = res }



-- SUBSCRIPTIONS


subscriptions : Model -> Sub Msg
subscriptions model =
    Sub.none



-- HTTP


getReservations : Cmd Msg
getReservations =
    Http.get
        { url = "http://localhost:3000/reservation"
        , expect = Http.expectJson GotReservations reservationDecoder
        }


getSystems : Cmd Msg
getSystems =
    Http.get
        { url = "http://localhost:3000/pnpsystem"
        , expect = Http.expectJson GotSystems systemDecoder
        }


reservationDecoder : Decoder (List Reservation)
reservationDecoder =
    list
        (map4
            Reservation
                (field "date" string)
                (field "hour" int)
                (field "dm" string)
                (field "system" string)
        )
    


systemDecoder : Decoder (List System)
systemDecoder =
    list 
        (map2
            System
                (field "name" string)
                (field "url" string)
        )

-- VIEW


view : Model -> Document Msg
view model =
    { title =
        "Reservierungs-App"
    , body =
        [ div [ class "columns is-centered theme-dark has-background-black-ter" ]
            [ case model.url.fragment of
                Just fragment ->
                    case fragment of
                        "Konfiguration" ->
                            let
                                systList =
                                    systemList model.loadingSystem
                            in
                            div [ class "px-6 column is-three-quarters-desktop" ]
                                [ applicationHeader
                                , buttonsSystemLibary model.loadingSystem
                                , viewSystemLibary systList
                                , viewModal model
                                ]

                        _ ->
                            div [ class "px-6 column is-three-quarters-desktop" ]
                                [ applicationHeader
                                , div [ class "container is-size-2 mt-5 has-text-info", style "height" "100vh" ] [ text "Nothing found" ]
                                ]

                Nothing ->
                    div [ class "container px-6 column is-three-quarters-desktop", style "height" "100vh" ]
                        [ applicationHeader
                        , buttons model.date model.loadingRes
                        , imageHeader model.date
                        , weekButtons model.date model.loadingRes
                        , roomImage model.status model.saved
                        , viewModal model
                        ]
            ]
        ]
    }


buttons : Date -> LoadingState -> Html Msg
buttons date loadingRes =
    div [ class "mb-4" ]
        [ button [ class "button is-primary mr-3" ]
            [ a [ class "has-text-black", onClick (OpenModal (OpenInputReservation date)), href "" ] [ text "Reservieren" ] ]
        , button [ class "button is-primary mr-3" ]
            [ a [ class "has-text-black", onClick (OpenModal OpenShowReservation), href "" ] [ text "Reservierung anzeigen" ] ]
        , button [ class "button is-link mr-3" ]
            [ a [ class "has-text-white", onClick (OpenModal OpenShowCalendar), href "" ] [ text "Kalender-Ansicht" ] ]
        , button [ class "button is-link mr-3" ]
            [ a [ class "has-text-white", href "#Konfiguration" ] [ text "System Bibliothek" ] ]
        , if loadingRes == Loading then
            button [ class "button is-warning is-loading mr-2" ]
                [ a [ onClick LoadReservations, href "" ] [] ]

          else
            button [ class "button is-warning mr-2" ]
                [ a [ class "has-text-black", onClick LoadReservations, href "" ] [ text "Laden" ] ]
        ]


applicationHeader : Html Msg
applicationHeader =
    div [ class "py-2 mb-4 mt-3" ]
        [ h1 [ class "title is-1" ] [ text "Reservierungs-App" ]
        , h2 [ class "title is-3" ] [ text "für Rollenspielrunden" ]
        ]


imageHeader : Date -> Html Msg
imageHeader date =
    div [ class "container is-widescreen is-flex-direction-row is-flex is-align-content-center is-justify-content-space-between has-text-centered py-2 px-4" ]
        [ button [ class "button is-info" ]
            [ a [ class "has-text-black", onClick (SetDate (add Days -1 date)), href "" ] [ text "Vorheriger Tag" ] ]
        , h3 [ class "title is-3" ]
            [ text (getWeekday date ++ ", " ++ printDate date) ]
        , button [ class "button is-info" ]
            [ a [ class "has-text-black", onClick (SetDate (add Days 1 date)), href "" ] [ text "Nächster Tag" ] ]
        ]


weekButtons : Date -> LoadingState -> Html Msg
weekButtons date loadingRes =
    div [ class "container is-widescreen is-flex-direction-row is-flex is-align-content-center is-justify-content-space-between has-text-centered py-2 px-4 field is-grouped" ]
        [ button [ class "button is-info" ]
            [ a [ class "has-text-black", onClick (SetDate (add Days -7 date)), href "" ] [ text "Vorherige Woche" ] ]
        , let
            weeks =
                weekDates date
                    |> groupsOf 7
          in
          Html.table [ style "border-collapse" "collapse", style "text-align" "right" ]
            [ Html.tbody
                []
                (weeks
                    |> List.map
                        (\weekdates ->
                            Html.tr
                                []
                                (weekdates
                                    |> List.map
                                        (\d ->
                                            let
                                                color =
                                                    if getStatus d loadingRes == Free then
                                                        "white"

                                                    else
                                                        "OrangeRed"

                                                background =
                                                    if d == date then
                                                        "lightskyblue"

                                                    else
                                                        "transparent"
                                            in
                                            Html.td
                                                [ class "is-clickable"
                                                , style "padding" "0.5em"
                                                , style "background" background
                                                , style "color" color
                                                , onClick (SetDate d)
                                                , onDoubleClick (OpenModal (OpenInputReservation d))
                                                ]
                                                [ Html.text (Date.day d |> String.fromInt)
                                                ]
                                        )
                                )
                        )
                )
            ]
        , button [ class "button is-info" ]
            [ a [ class "has-text-black", onClick (SetDate (add Days 7 date)), href "" ] [ text "Nächste Woche" ] ]
        ]


roomImage : Status -> Reservation -> Html Msg
roomImage status saved =
    case status of
        Taken ->
            let
                logo =
                    getLogo saved.system
            in
            figure [ class "image is-3by2" ]
                [ Svg.svg
                    [ Svg.Attributes.width "100%"
                    , Svg.Attributes.height "100%"
                    , Svg.Attributes.viewBox "0 0 600 400"
                    ]
                    [ Svg.image
                        [ Svg.Attributes.width "100%"
                        , x "0"
                        , y "0"
                        , Svg.Attributes.xlinkHref "FullRoom.jpeg"
                        ]
                        []
                    , Svg.text_
                        [ x "305"
                        , y "215"
                        , Svg.Attributes.pointerEvents "none"
                        , Svg.Attributes.textAnchor "middle"
                        , Svg.Attributes.fontSize "15"
                        , Svg.Attributes.fontWeight "bolder"
                        , Svg.Attributes.fontFamily "cursive"
                        ]
                        [ Svg.text saved.dm ]
                    , Svg.image
                        [ Svg.Attributes.width "200"
                        , x "100"
                        , y "70"
                        , Svg.Attributes.xlinkHref logo
                        ]
                        []
                    , Svg.text_
                        [ x "152"
                        , y "190"
                        , Svg.Attributes.pointerEvents "none"
                        , Svg.Attributes.fontSize "12"
                        , Svg.Attributes.fontWeight "bolder"
                        , Svg.Attributes.fontFamily "Georgia"
                        ]
                        [ Svg.text ("Beginn: " ++ String.fromInt saved.hour ++ " Uhr") ]

                    -- , Svg.polygon [ Svg.Attributes.points "0,0 600,0 600,400 0,400", Svg.Attributes.fill "none", Svg.Attributes.stroke "red", Svg.Attributes.strokeWidth "5" ] []
                    ]
                ]

        Free ->
            figure [ class "image is-3by2" ]
                [ Svg.svg
                    [ Svg.Attributes.width "100%"
                    , Svg.Attributes.height "100%"
                    , Svg.Attributes.viewBox "0 0 600 400"
                    ]
                    [ Svg.image
                        [ Svg.Attributes.width "100%"
                        , x "0"
                        , y "0"
                        , Svg.Attributes.xlinkHref "EmptyRoom.jpeg"
                        ]
                        []

                    -- , Svg.polygon [ Svg.Attributes.points "0,0 600,0 600,400 0,400", Svg.Attributes.fill "none", Svg.Attributes.stroke "red", Svg.Attributes.strokeWidth "5" ] []
                    ]
                ]

        Unknown ->
            figure [ class "image is-3by2" ]
                [ Svg.svg
                    [ Svg.Attributes.width "100%"
                    , Svg.Attributes.height "100%"
                    , Svg.Attributes.viewBox "0 0 600 400"
                    ]
                    [ Svg.image
                        [ Svg.Attributes.width "100%"
                        , x "0"
                        , y "0"
                        , Svg.Attributes.xlinkHref "EmptyRoom.jpeg"
                        ]
                        []

                    -- , Svg.polygon [ Svg.Attributes.points "0,0 600,0 600,400 0,400", Svg.Attributes.fill "none", Svg.Attributes.stroke "red", Svg.Attributes.strokeWidth "5" ] []
                    ]
                ]


viewModal : Model -> Html Msg
viewModal model =
    let
        _ =
            Debug.log "viewModal" model.modal
    in
    case model.modal of
        Nothing ->
            span [] []

        Just modalState ->
            div [ class "theme-light" ]
                [ case modalState of
                    ShowCalendar ->
                        showSavedAsCalendar model

                    ShowReservation ->
                        showReservationInModal model.saved

                    InputReservation reservation loadingS ->
                        inputReservationInModal reservation loadingS

                    ShowSystem system ->
                        showSystemInModal system

                    InputSystem system ->
                        inputSystemInModal system
                ]


modalFooter : List (Html Msg) -> Html Msg
modalFooter modalButtons =
    div []
        (modalButtons
            ++ [ button [ class "button is-danger" ] [ a [ class "has-text-white", onClick CloseModal, href "" ] [ text "Schliessen" ] ] ]
        )


showReservationInModal : Reservation -> Html Msg
showReservationInModal saved =
    div [ class "modal is-active" ]
        [ div [ class "modal-background" ] []
        , div [ class "modal-card" ]
            [ div [ class "modal-card-head" ]
                [ h2 [ class "title is-3" ] [ text "Aktuelle Reservierung" ] ]
            , section [ class "modal-card-body" ]
                [ div [ class "" ]
                    [ div []
                        [ span [] [ text ("Datum: " ++ saved.date) ] ]
                    , div []
                        [ span [] [ text ((" Uhrzeit: " ++ String.fromInt saved.hour) ++ " Uhr") ]
                        ]
                    , div [] [ text ("Spielleiter: " ++ saved.dm) ]
                    , div [] [ text ("System: " ++ saved.system) ]
                    ]
                ]
            , footer [ class "modal-card-foot" ]
                [ modalFooter []
                ]
            ]
        ]


showSavedAsCalendar : Model -> Html Msg
showSavedAsCalendar model =
    div [ class "modal is-active" ]
        [ div [ class "modal-background" ] []
        , div [ class "modal-card" ]
            [ div [ class "modal-card-head" ]
                [ h2 [ class "title is-3" ] [ text "Kalender-Ansicht" ]
                ]
            , section [ class "modal-card-body" ]
                [ viewCalendar model.date model.loadingRes
                ]
            , footer [ class "modal-card-foot" ]
                [ modalFooter []
                ]
            ]
        ]


inputReservationInModal : Reservation -> SystemLoadingState -> Html Msg
inputReservationInModal r loadingSyst =
    div [ class "modal is-active" ]
        [ div [ class "modal-background" ] []
        , div [ class "modal-card" ]
            [ div [ class "modal-card-head" ]
                [ h2 [ class "modal-card-title title is-3" ] [ text "Bitte Daten eingeben" ]
                ]
            , section [ class "modal-card-body" ]
                [ inputReservation r loadingSyst
                ]
            , footer [ class "modal-card-foot" ]
                [ modalFooter
                    [ button [ class "button is-link mr-4" ]
                        [ a [ class "has-text-white", onClick (OpenModal (Add r)) ] [ text "Reservieren" ]
                        ]
                    ]
                ]
            ]
        ]


inputReservation : Reservation -> SystemLoadingState -> Html Msg
inputReservation r loadingSyst =
    let
        changeDate newDate =
            OpenModal
                (Change
                    { r
                        | date =
                            case Date.fromIsoString newDate of
                                Ok d ->
                                    Date.toIsoString d

                                Err _ ->
                                    "2019-01-01"
                    }
                )

        changeHour newH =
            OpenModal (Change { r | hour = Maybe.withDefault 17 (String.toInt newH) })

        changeDM newDM =
            OpenModal (Change { r | dm = newDM })

        changeSystem newSystem =
            OpenModal (Change { r | system = newSystem })

        systList =
            getSystemNames loadingSyst
    in
    fieldset []
        [ div [ class "has-text-weight-semibold is-size-5" ]
            [ text "Datum"
            , input [ class "input", type_ "text", value r.date, onInput changeDate ] []
            ]
        , div [ class "has-text-weight-semibold is-size-5" ]
            [ text "Uhrzeit"
            , input [ class "input", type_ "text", value (String.fromInt r.hour), onInput changeHour ] []
            ]
        , div [ class "has-text-weight-semibold is-size-5" ]
            [ text "Spielleiter"
            , input [ class "input", type_ "text", value r.dm, onInput changeDM, placeholder "Spielleiter" ] []
            ]
        , div [ class "has-text-weight-semibold is-size-5" ]
            [ text "System"
            , input [ class "input", type_ "text", value r.system, onInput changeSystem, placeholder "System" ] []
            ]

        --, div [ ]
        --    [ div [ class "has-text-weight-semibold is-size-5" ] [ text "System" ]
        --    , div [ class "select" ]
        --        [ select [ placeholder "System" ]
        --            ( systList
        --                |> List.map
        --                    (\s -> option [] [ text s ]
        --                    )
        --            )
        --        ]
        --    ]
        ]
